/*
 * Copyright (C) 1997-2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "para.h"


#define MMD_INFO_SIZE 16384

/* data needed to authenticate user */
struct user{
	char name[MAXLINE];
	char pubkey_file[_POSIX_PATH_MAX];
	unsigned int perms;
};

/* contains data for arbitrary command */
struct command {
	char *name;
	unsigned int perms;
	char *description;
	char *usage;
	char *help;
	int (*handler)(int, int, char **);
};

struct sender_command_data{
	int cmd_num; /* >= 0 indicates that a server cmd is already queued */
	int sender_num;
	struct sockaddr_in self;
	struct in_addr addr; /* allow/deny, add/remove */
	int netmask; /* allow/deny */
	int port; /*tcp or udp */
};

/* there's one struct of this type which lives in shared memory
 * for communication between the server instances. Access to this
 * area is serialized via mmd_lock()/mmd_unlock(). There are two
 * reasons for a variable to be included here:
 * (a) at least one command (i.e. child of the server) must be able to
 *     change its value
 * or
 * (b) The contents are listed in the stat command and have to be up to date
 */
struct misc_meta_data{
/* audio file info */
	long unsigned int size;
	char filename[_POSIX_PATH_MAX];
	time_t mtime;
/* afs status info */
	int audio_format;
	unsigned int afs_status_flags, new_afs_status_flags;
	long unsigned chunks_sent, chunks_total, repos_request, current_chunk;
	long offset; /* milliseconds */
	int seconds_total;
	struct timeval stream_start;
	char audio_file_info[MMD_INFO_SIZE];
/* server info */
	unsigned int events;
	unsigned int num_played;
	unsigned int num_commands;
	unsigned int num_connects;
	unsigned int active_connections;
	pid_t server_pid;
/* dbtool */
	char dbinfo[MMD_INFO_SIZE];
	int dbt_num; /* number of current dbtool */
	int dbt_change; /* requests dbtool change if non-negative */
/* sender */
	struct sender_command_data sender_cmd_data;
};


int handle_connect(int fd, struct sockaddr_in *addr);
void mmd_unlock(void);
void mmd_lock(void);
